#ifndef PKMNLIB_NATURELIBRARY_HPP
#define PKMNLIB_NATURELIBRARY_HPP

#include <Arbutils/Memory/Memory.hpp>
#include <Arbutils/Random.hpp>
#include <CreatureLib/Library/Exceptions/CreatureException.hpp>
#include "Nature.hpp"

namespace PkmnLib::Library {
    class NatureLibrary {
    private:
        std::unordered_map<ArbUt::StringView, std::unique_ptr<const Nature>> _items;

    public:
        explicit NatureLibrary(size_t size = 32) noexcept : _items(size) {}

        ~NatureLibrary() = default;

        inline void LoadNature(const ArbUt::StringView& name, const Nature* nature) {
            _items.insert({name, std::unique_ptr<const Nature>(nature)});
        }

        inline ArbUt::BorrowedPtr<const Nature> GetNatureByName(const ArbUt::StringView& name) const {
            return _items.at(name);
        }

        inline const ArbUt::StringView& GetRandomNatureName(ArbUt::Random rand = ArbUt::Random()) const {
            auto i = rand.Get(_items.size());
            return std::next(std::begin(_items), i)->first;
        }

        inline ArbUt::StringView GetNatureName(ArbUt::BorrowedPtr<const Nature> nature) {
            for (const auto& v : _items) {
                if (v.second.get() == nature.GetRaw()) {
                    return v.first;
                }
            }
            throw ArbUt::Exception("Nature not found.");
        }

        size_t GetNatureCount() const noexcept { return _items.size(); }
        inline const ArbUt::StringView& GetNatureFromIndex(size_t index) const {
            return std::next(std::begin(_items), index)->first;
        }
    };
}

#endif // PKMNLIB_NATURELIBRARY_HPP
