#ifndef PKMNLIB_POKEMONLIBRARY_HPP
#define PKMNLIB_POKEMONLIBRARY_HPP

#include <CreatureLib/Library/DataLibrary.hpp>
#include "Items/ItemLibrary.hpp"
#include "LibrarySettings.hpp"
#include "Moves/MoveLibrary.hpp"
#include "Natures/NatureLibrary.hpp"
#include "Species/SpeciesLibrary.hpp"

namespace PkmnLib::Library {
    class PokemonLibrary final : public CreatureLib::Library::DataLibrary {
    private:
        std::unique_ptr<const NatureLibrary> _natures;

    public:
        ~PokemonLibrary() override = default;

        PokemonLibrary(PkmnLib::Library::LibrarySettings* settings, SpeciesLibrary* species, MoveLibrary* moves,
                       ItemLibrary* items, CreatureLib::Library::GrowthRateLibrary* growthRates,
                       CreatureLib::Library::TypeLibrary* typeLibrary,
                       CreatureLib::Library::TalentLibrary* talentLibrary, NatureLibrary* natures)
            : DataLibrary(settings, species, moves, items, growthRates, typeLibrary, talentLibrary), _natures(natures) {
        }

        const std::unique_ptr<const PkmnLib::Library::LibrarySettings>& GetSettings() const {
            return (const std::unique_ptr<const LibrarySettings>&)CreatureLib::Library::DataLibrary::GetSettings();
        }

        [[nodiscard]] const std::unique_ptr<const SpeciesLibrary>& GetSpeciesLibrary() const {
            return (const std::unique_ptr<const SpeciesLibrary>&)CreatureLib::Library::DataLibrary::GetSpeciesLibrary();
        }

        [[nodiscard]] const std::unique_ptr<const MoveLibrary>& GetMoveLibrary() const {
            return (const std::unique_ptr<const MoveLibrary>&)CreatureLib::Library::DataLibrary::GetAttackLibrary();
        }

        [[nodiscard]] const std::unique_ptr<const ItemLibrary>& GetItemLibrary() const {
            return (const std::unique_ptr<const ItemLibrary>&)CreatureLib::Library::DataLibrary::GetItemLibrary();
        }

        [[nodiscard]] ArbUt::BorrowedPtr<const NatureLibrary> GetNatureLibrary() const noexcept { return _natures; }
    };
}

#endif // PKMNLIB_POKEMONLIBRARY_HPP
