#include "RegisterGrowthRateTypes.hpp"
#include <Arbutils/Ensure.hpp>
#include <CreatureLib/Library/GrowthRates/GrowthRate.hpp>
#include <CreatureLib/Library/GrowthRates/GrowthRateLibrary.hpp>

void RegisterGrowthRateTypes::Register(asIScriptEngine* engine) {
    RegisterGrowthRateType(engine);
    RegisterGrowthRateLibrary(engine);
}
void RegisterGrowthRateTypes::RegisterGrowthRateType(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("GrowthRate", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("GrowthRate", "uint8 CalculateLevel(uint experience) const",
                                     asMETHOD(CreatureLib::Library::GrowthRate, CalculateLevel), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("GrowthRate", "uint CalculateExperience(uint8 level) const",
                                     asMETHOD(CreatureLib::Library::GrowthRate, CalculateExperience), asCALL_THISCALL);
    Ensure(r >= 0);
}

static uint8_t CalculateLevel(const CreatureLib::Library::GrowthRateLibrary* obj, const ArbUt::StringView& str,
                              uint32_t experience) {
    return obj->CalculateLevel(str, experience);
}
static uint32_t CalculateExperience(const CreatureLib::Library::GrowthRateLibrary* obj, const ArbUt::StringView& str,
                                    level_int_t level) {
    return obj->CalculateExperience(str, level);
}

void RegisterGrowthRateTypes::RegisterGrowthRateLibrary(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("GrowthRateLibrary", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("GrowthRateLibrary",
                                     "uint8 CalculateLevel(const constString &in growthRate, uint experience) const",
                                     asFUNCTION(CalculateLevel), asCALL_CDECL_OBJLAST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod(
        "GrowthRateLibrary", "uint CalculateExperience(const constString &in growthRate, uint8 experience) const",
        asFUNCTION(CalculateExperience), asCALL_CDECL_OBJLAST);
    Ensure(r >= 0);
}
