#include "RegisterItemTypes.hpp"
#include "../../../../Library/Items/Item.hpp"
#include "../../../../Library/Items/ItemLibrary.hpp"
#include "../HelperFile.hpp"

void RegisterItemTypes::Register(asIScriptEngine* engine) {
    RegisterItemCategoryEnum(engine);
    RegisterBattleItemCategoryEnum(engine);
    RegisterItemType(engine);
    RegisterItemLibrary(engine);
}

void RegisterItemTypes::RegisterItemCategoryEnum(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterEnum("ItemCategory");
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("ItemCategory", "Misc", (int)CreatureLib::Library::ItemCategory::MiscItem);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("ItemCategory", "Pokeball", (int)CreatureLib::Library::ItemCategory::CaptureDevice);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("ItemCategory", "Medicine", (int)CreatureLib::Library::ItemCategory::Medicine);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("ItemCategory", "Berry", (int)CreatureLib::Library::ItemCategory::Berry);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("ItemCategory", "TM", (int)CreatureLib::Library::ItemCategory::MoveLearner);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("ItemCategory", "VariantChanger",
                                  (int)CreatureLib::Library::ItemCategory::VariantChanger);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("ItemCategory", "KeyItem", (int)CreatureLib::Library::ItemCategory::KeyItem);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("ItemCategory", "Mail", (int)CreatureLib::Library::ItemCategory::Mail);
    Ensure(r >= 0);
}

void RegisterItemTypes::RegisterBattleItemCategoryEnum(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterEnum("BattleItemCategory");
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("BattleItemCategory", "None", (int)CreatureLib::Library::BattleItemCategory::None);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("BattleItemCategory", "Healing",
                                  (int)CreatureLib::Library::BattleItemCategory::Healing);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("BattleItemCategory", "StatusHealing",
                                  (int)CreatureLib::Library::BattleItemCategory::StatusHealing);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("BattleItemCategory", "Pokeball",
                                  (int)CreatureLib::Library::BattleItemCategory::CaptureDevice);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("BattleItemCategory", "Misc",
                                  (int)CreatureLib::Library::BattleItemCategory::MiscBattleItem);
    Ensure(r >= 0);
}

static bool HasFlag(const PkmnLib::Library::Item* obj, const ArbUt::BasicStringView& str) { return obj->HasFlag(str); }

void RegisterItemTypes::RegisterItemType(asIScriptEngine* engine) {
    int r = engine->RegisterObjectType("Item", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Ensure(r >= 0);

    REGISTER_GETTER("Item", "const constString& get_Name() const property", CreatureLib::Library::Item, GetName);
    REGISTER_GETTER("Item", "ItemCategory get_Category() const property", CreatureLib::Library::Item, GetCategory);
    REGISTER_GETTER("Item", "BattleItemCategory get_BattleCategory() const property", CreatureLib::Library::Item,
                    GetBattleCategory);
    REGISTER_GETTER("Item", "int get_Price() const property", CreatureLib::Library::Item, GetPrice);

    r = engine->RegisterObjectMethod("Item", "bool HasFlag(const constString &in flag) const", asFUNCTION(HasFlag),
                                     asCALL_CDECL_OBJLAST);
    Ensure(r >= 0);
}
void RegisterItemTypes::RegisterItemLibrary(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("ItemLibrary", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("ItemLibrary", "const Item@ Get(const constString &in name) const",
                                     asMETHOD(PkmnLib::Library::ItemLibrary, Get), asCALL_THISCALL);
    Ensure(r >= 0);
}
